package main

import (
	"flag"
	"log"
	"net/http"

	"douglasthrift.net/dtrobots/gamepad"

	"github.com/ev3go/ev3dev"
)

func main() {
	var (
		device      string
		listen      string
		leftPort    string
		leftDriver  string
		rightPort   string
		rightDriver string
		mode        driveMode
	)

	flag.StringVar(&device, "gamepad", "/dev/input/event0", "the gamepad device path")
	flag.StringVar(&listen, "listen", "localhost:8080", "the HTTP listen address and port")
	flag.StringVar(&leftPort, "motor-left-port", "spi0.1:MC", "the left motor port")
	flag.StringVar(&leftDriver, "motor-left-driver", "lego-nxt-motor", "the left motor driver")
	flag.StringVar(&rightPort, "motor-right-port", "spi0.1:MB", "the right motor port")
	flag.StringVar(&rightDriver, "motor-right-driver", "lego-nxt-motor", "the right motor driver")
	flag.Var(&mode, "mode", "the drive mode (default \"tank\")")
	flag.Parse()

	g, err := gamepad.NewGamepad(device, gamepad.All)
	if err != nil {
		log.Fatalf("error creating gamepad: %v", err)
	}

	leftMotor, err := ev3dev.TachoMotorFor(leftPort, leftDriver)
	if err != nil {
		log.Fatalf("error detecting left motor: %v", err)
	}
	log.Printf("detected left motor: %v", leftMotor)

	rightMotor, err := ev3dev.TachoMotorFor(rightPort, rightDriver)
	if err != nil {
		log.Fatalf("error detecting right motor: %v", err)
	}
	log.Printf("detected right motor: %v", rightMotor)

	go func() {
		log.Fatalf("error controlling rover: %v", control(mode, g, leftMotor, rightMotor))
	}()
	go func() {
		log.Fatalf("error reading from gamepad: %v", g.Read())
	}()

	log.Print("HTTP server started")
	log.Fatalf("error listening and serving: %v", http.ListenAndServe(listen, nil))
}
